// ESP-now-data_station_SHT20.ino
// ESP32 @ IDE 1.8.16

#define BOARD_ID 1

#include <esp_now.h>
#include <esp_wifi.h>
#include <WiFi.h>
#include <Wire.h>
#include "uFire_SHT20.h"

uFire_SHT20 sht20;
uint8_t broadcastAddress[] = {0x00,0x00,0x00,0x00,0x00,0x00};  // Rx Board 00

typedef struct struct_message {int id; float temp; float hum; int readingId;} struct_message;
struct_message myData;

unsigned long previousMillis = 0;   // Stores last time temperature was published
const long interval = 10000;        // Interval at which to publish sensor readings in ms
unsigned int readingId = 0;
constexpr char WIFI_SSID[] = "XXXXXXXXXXX"; // Insert your SSID 

int32_t getWiFiChannel(const char *ssid)
{ if (int32_t n = WiFi.scanNetworks()) 
  { for (uint8_t i=0; i<n; i++) 
    { if (!strcmp(ssid, WiFi.SSID(i).c_str()))  return WiFi.channel(i); }
    return 0;
  }
}

// callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}

esp_now_peer_info_t peerInfo;

void setup() 
{ Serial.begin(115200);
  Wire.begin(); sht20.begin();
  WiFi.mode(WIFI_STA);
  int32_t channel = getWiFiChannel(WIFI_SSID);
  WiFi.printDiag(Serial); // Uncomment to verify channel number before
  esp_wifi_set_promiscuous(true);
  esp_wifi_set_channel(channel, WIFI_SECOND_CHAN_NONE);
  esp_wifi_set_promiscuous(false);
  WiFi.printDiag(Serial); // Uncomment to verify channel change after
  if (esp_now_init() != ESP_OK) { Serial.println("Error initializing ESP-NOW"); return; }
  esp_now_register_send_cb(OnDataSent);
  
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.encrypt = false;
  if (esp_now_add_peer(&peerInfo) != ESP_OK)
  { Serial.println("Failed to add peer"); return;
  }
}
 
void loop() 
{ unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) 
  { previousMillis = currentMillis;
    myData.id = BOARD_ID;
    sht20.measure_all();
    myData.temp = sht20.tempC;    Serial.println(sht20.tempC);
    myData.hum = sht20.RH;        Serial.println(sht20.RH);
    myData.readingId = readingId++;
    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &myData, sizeof(myData));
    if (result == ESP_OK) Serial.println("Data Sent"); else Serial.println("Send Fail");
  }
}
